### Building

The documentation for `geoplot` is in the form of a fairly complicated `sphinx` website, with a bunch of 
enabled extensions (most prominently `autosummary`) and a palette swap (for the `sphinx-bootstrap-theme`).

There are four kinds of pages on the website:

1. **Front-matter pages**. These are things like the home page, the API reference, and so on. I write these in 
ReStructed Text directly; they're hosted at the top level of the `/docs` subdirectory.
2. **Method API references**. The API reference subpages are built using `autosummary` (although the API reference 
front page itself is a raw RST file). The contents of those files are generated from the docstrings of the methods in
 the Python source. These include embedded images; the images are generated by running a Jupyter notebook located in 
 the `/notebooks` subdirectory called `generate-api-reference-images`. `generate-api-reference-images.py` in the 
 `/scripts` subdirectory is a two-line Python script (using `nbconvert`) which exports these from the command line (note 
 that rerunning the entire notebook takes a while).
3. **Examples**. The examples are written to the `/docs/examples` subdirectory. The core of each example is a Python 
file which saves its output to a `png` or `html` file. The companion (hand-written) `rst` file is just a tiny bit of 
syntactic sugar on top of these two files. These are then linked to the Gallery page by hand.
4. **Tutorial pages**. These are the most involved. Tutorial pages aside from the front page are written in Jupyter 
notebooks and then exported to ReStructured Text using `nbconvert.` This output is then passed on to the 
`/docs/tutorial` subdirectory. `export-tutorial.py` in `/scripts` handles this export process, again using 
`nbconvert`.

   Note that the tutorial export as written isn't well-formatted `RST` because it links to images via paths which 
   don't exist. This means that when we convert the `RST` to `HTML`, we'll get a bunch of dead links in place of our 
   images. This is the fault of `nbconvert`, which weirdly exports images to `/docs/docs/tutorial/` whilst exporting 
   text to `/docs/tutorial/`, but then doesn't reference that correctly.
   
   I could theoretically fix this with some munging in the export script (and perhaps I should; or even better, 
   report this as a bug?), but thus far I've found it easiest to fix this problem on the front-end (by rerouting 
   what file path the image URL references). Here's the responsible bit of code in my `flask` `app.py`:

    ```python
       
       @app.route('/geoplot/<path:path>')
       def serve_geoplot_documentation(path):
           # Exported tutorial images are placed in the docs/tutorial subdirectory. All other assets are placed in the
           # _build/html subdirectory. We serve each separately.
       
           # Regular assets.
           if 'docs/docs/' not in path:
               return send_from_directory('geoplot_docs', '_build/html/' + path)
           # Tutorial images.
           else:
               print(path.replace("docs/docs/", "geoplot_docs/docs/"))
               return send_from_directory('geoplot_docs', path.replace("docs/docs/", "docs/"))
    ```
    
    All this isn't ideal, but it works for now.
    
Once you're done making changes to documentation, `cd` to `/docs` and run `make html` (or `make html clean`, if 
you've made CSS changes) to rebuild the docs.

### Publishing

I publish the docs as a set of directory-hosted static pages in a `/geoplot` sub-path on my personal website. To get 
that hosted I copy-paste the entire `/docs` directory into a renamed `/geoplot_docs` top-level directory on the 
`mysite` repository (`docs` alone is ambiguous).